/******************************************************************************/
/* cmfopts.c       CICS on Open Systems Monitor Data Formatter   Rags Gardner */
/*                                                                            */
/* Rags Int., Inc.                                              December 1999 */
/* (c) RISC Application Growth Solutions International, Incorporated  1999    */
/*             (817) 267-2554       rags@compuserve.com                       */
/*                                                                            */
/******************************************************************************/
#ifdef WIN32
   #include <windows.h>                                            /* windows */
   #include <stdarg.h>                                 /* argument processing */
   #include <getopt.h>                                 /* argument processing */
#endif
#include <stdio.h>                                       /* Unix standard I/O */
#include <ctype.h>                                     /* character functions */
#include <sys/types.h>                                 /* standard data types */
#include <memory.h>                                                 /* memory */
#include <string.h>                                                /* strings */
#include <cics_types.h>                                    /* CICS data types */
#include <cicsmfmt.h>                             /* monitor formatter header */
#include "cmfreport.h"                                    /* cmfreport header */

/*----------------------------------------------------------------------------*/
/* global storage areas                                                       */
/*----------------------------------------------------------------------------*/
extern int    LineLth;                                 /* characters per line */
extern int    ReportWidth;              /* line size based on selected fields */
extern int    PageLth;                                      /* lines per page */
extern int    FieldCount;                         /* count of selected fields */
extern int    IncludeCount;                        /* count of include fields */
extern int    ExcludeCount;                        /* count of exclude fields */
extern int    TitleLines;                             /* count of title lines */
extern int    TimeInSec;      /* timers reported in seconds.milliseconds flag */
extern int    PrtDetail;                     /* suppress detail, summary only */
extern int    SummaryRF;                    /* accumulate summary report flag */
extern int    SummaryF;                       /* suppress summary totals flag */
extern int    PrtCount;              /* print counters associated with clocks */
extern int    GMT;                        /* show time stamps in local or GMT */
extern int    MonFields;             /* number of monitor field types defined */
extern char   ProgName[];                                /* this program name */
extern struct FieldDesc_t FormatMap[];     /* cmf field formatting structures */
extern struct SummaryTable_t Table;   /* table containing accumulated records */

/******************************************************************************/
/* cmfopts()                             Parse command line arguments         */
/******************************************************************************/
int cmfopts(int    argc,                                    /* argument count */
               char  *argv[],                  /* user argument input strings */
               char   MonDataFile[],                    /* CMF data file name */
               char   TopTitle[],                           /* top title text */
               struct FieldList_t   FieldList[],    /* selected report fields */
               struct ExcIncList_t IncludeList[],         /* include criteria */
               struct ExcIncList_t ExcludeList[]          /* exclude criteria */
              )
{
#ifdef WIN32
  char             *optarg;                               /* option arguments */
#else
  extern char      *optarg;                               /* option arguments */
#endif
  extern int        optind;                             /* sub argument count */
  int               option;                   /* command arguments (-options) */
  int               i, i2, i3;                               /* work integers */
  int               SubParmCt=0;                         /* sub operand count */
  int               SubParmCt2=0;                        /* sub operand count */
  int               empWork=0;              /* work area for field emp number */
  int               ErrOpt=FALSE;                    /* option error detected */
  int               FileNameProcessed=FALSE;         /* option error detected */
  char              ParmBuf[LIST_SZ*2][30]={'\0'};           /* sub arguments */
  char              ParmBuf2[2][30]={'\0'};                  /* sub arguments */
    /*----------------------------------------------------*/
    /* Validate the command line arguments                */
    /*----------------------------------------------------*/
#ifdef WIN32
    while ((option = GetOption(argc, argv,      /* get command line arguments */
          "?cGSTqxa:e:f:i:l:s:t:w:", &optarg)) != 0)
#else
    while ((option = getopt(argc, argv,         /* get command line arguments */
          "?cGSTqxa:e:f:i:l:s:t:w:")) != EOF)
#endif
    {
       switch (option)
       {

       /*---------------------------------------------------------*/
       /*- accumulate major/minor summaries                      -*/
       /*- (parse major/minor fields from optargs) !!!           -*/
       /*- (validate major/minor are string fields)              -*/
       /*- (identify major/minor in FromatMap)                   -*/
       /*---------------------------------------------------------*/
       case   'a':
          SummaryRF=TRUE;
          SubParmCt=sscanf(optarg, "%[^,], %[^,], %[^,]",
                           ParmBuf[0], ParmBuf[1], ParmBuf[2]);
#ifdef DEBUG
          fprintf(stderr, "[%s]: Accumulating for Major/Minor columns "
                  "\"%s\"/\"%s\"\n", ProgName, ParmBuf[0], ParmBuf[1]);
#endif
          if ((SubParmCt == 1) || (SubParmCt == 2))
          {
             for (i=0; i<SubParmCt; i++)             /* for each sub argument */
             {                             
                if (isdigit(ParmBuf[i][0]))            /* if a digit, take it */
                {
                  empWork=atoi(ParmBuf[i]);
                } else {                  /* else, convert name to emp number */
                  empWork=Field2Num(&ParmBuf[i][0]);
                  if (empWork < 0) {
                     fprintf(stderr,
                        "[%s]: Accumulate - unrecognized emp \"%s\"\n",
                        ProgName, &ParmBuf[i][0]);
                     ErrOpt=TRUE;                   /* put out a help message */
                     break;
                  }
                } /* end (isdigit) */
                if (i == 0)                          /* save the major column */
                   Table.Major = emp2Fieldp(empWork, &ParmBuf[i][0]);
                else                                 /* save the minor column */
                   Table.Minor = emp2Fieldp(empWork, &ParmBuf[i][0]);
             } /* end for (SubParmCt) */

             switch (Table.Major->fieldType)
             {
                case CICS_EMP_TDQ_DETAIL_TYPE_ABEND_CODE:
                case CICS_EMP_TDQ_DETAIL_TYPE_STRING:
                     break;                   /* break on valid data types */
                default:
                     fprintf(stderr, "[%s]: Accumulate - major column (%s) "
                             "must be a string data type\n",
                             ProgName, &ParmBuf[0][0]);
                     ErrOpt=TRUE;        /* message for invalid data types */
                     break;
             }
             if (Table.Minor != NULL)            /* validate the minor column */
             {
                switch (Table.Minor->fieldType)
                {
                   case CICS_EMP_TDQ_DETAIL_TYPE_ABEND_CODE:
                   case CICS_EMP_TDQ_DETAIL_TYPE_STRING:
                        break;                   /* break on valid data types */
                   default:
                        fprintf(stderr, "[%s]: Accumulate - minor column (%s) "
                                "must be a string data type\n",
                                ProgName, &ParmBuf[1][0]);
                        ErrOpt=TRUE;        /* message for invalid data types */
                        break;
                }
                /*----------------------------------------*/
                /* insure that the major and minor fields */
                /* are not identical                      */
                /*----------------------------------------*/
                if (Table.Major->empNum == Table.Minor->empNum) {
                   fprintf(stderr, "[%s]: Accumulate - major and minor columns"
                           " must differ (%s)/(%s)\n", ProgName,
                           &ParmBuf[0][0], &ParmBuf[1][0]);
                   ErrOpt=TRUE;
                }
             }
          } else {
             /*------------------------------------------------------------*/
             /* Error: Major and Minor fields were not correctly specified */
             /*------------------------------------------------------------*/
             fprintf(stderr, "[%s]: Major and Minor fields (%s) "
                             "are invalid!\n", ProgName, optarg);
             ErrOpt=TRUE;                           /* put out a help message */
          } /* end if (SubParmCt) */
          break;

       /*-------------------------------------------------*/
       /*- print counters associated with clocks         -*/
       /*-------------------------------------------------*/
       case   'c':
          PrtCount=TRUE;
          break;

       /*-------------------------------------------------*/
       /*- suppress printing summary accumulators flag   -*/
       /*-------------------------------------------------*/
       case   'q':
          SummaryF=FALSE;
          break;

       /*-------------------------------------------------*/
       /*- show time stamps in GMT                       -*/
       /*-------------------------------------------------*/
       case   'G':
          GMT=TRUE;
          break;

       /*-------------------------------------------------*/
       /*- show timer values in seconds.milliseconds     -*/
       /*-------------------------------------------------*/
       case   'S':
          TimeInSec=TRUE;
          break;

       /*-------------------------------------------------*/
       /*- suppress field title lines                    -*/
       /*-------------------------------------------------*/
       case   'T':
          TitleLines=0;
          break;

       /*-------------------------------------------------*/
       /*- top title user text                           -*/
       /*-------------------------------------------------*/
       case   't':
          sprintf(TopTitle, "%s", optarg);
          TitleLines++;                     /* incriment count of title lines */
          break;

       /*-------------------------------------------------*/
       /*- lines per page                                -*/
       /*-------------------------------------------------*/
       case   'l':
          PageLth=atoi(optarg);
          break;

       /*-------------------------------------------------*/
       /*- Suppress detail lines (summary only)          -*/
       /*-------------------------------------------------*/
       case   'x':
          PrtDetail=FALSE;
          break;

       /*-------------------------------------------------*/
       /*- characters per line                           -*/
       /*-------------------------------------------------*/
       case   'w':
          LineLth=atoi(optarg);
          if (LineLth < 80 || LineLth > PRT_MAX) {
              fprintf(stderr, "Print LineLth(%i) must be between 80 & %i\n",
                       LineLth, PRT_MAX);
              ErrOpt=TRUE;                          /* put out a help message */
          }
          break;

       /*-------------------------------------------------*/
       /*- path and name of the CMF input data file      -*/
       /*-------------------------------------------------*/
       case   'f':
          strcpy((char*)MonDataFile, optarg);
          FileNameProcessed=TRUE;
          break;

       /*-------------------------------------------------*/
       /*- get a list of excluded fields and text        -*/
       /*-------------------------------------------------*/
       case   'e':
          SubParmCt=sscanf(optarg,
                           "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],"
                           "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],"
                           "%[^,],%[^,],%[^,],%[^,]",
                           ParmBuf[0],  ParmBuf[1],  ParmBuf[2],  ParmBuf[3],
                           ParmBuf[4],  ParmBuf[5],  ParmBuf[6],  ParmBuf[7],
                           ParmBuf[8],  ParmBuf[9],  ParmBuf[10], ParmBuf[11],
                           ParmBuf[12], ParmBuf[13], ParmBuf[14], ParmBuf[15],
                           ParmBuf[16], ParmBuf[17], ParmBuf[18], ParmBuf[19]);
          if (ExcludeCount+SubParmCt > LIST_SZ) {
             fprintf(stderr, "[%s]: ExcludeList list cannot exceed %d items!\n",
                ProgName, LIST_SZ);
             ErrOpt=TRUE;                           /* put out a help message */
             break;
          } 
          i2=ExcludeCount;                /* next unused slot in Exclude list */
          for (i=0; i<(SubParmCt); i++)                  /* for each argument */
          {                             
             SubParmCt2=sscanf(ParmBuf[i], "%[^=]=%[^=]=%[^=]",
                               ParmBuf2[0], ParmBuf2[1], ParmBuf2[2]);
             if (SubParmCt2 != 2) {
                fprintf(stderr, "[%s]: Exclude sublist must have one \"=\","
                        " %i found\n",
                        ProgName, (SubParmCt2-1));
                ErrOpt=TRUE;                        /* put out a help message */
             }
             if (ValidateIE(&ExcludeList[i2], ParmBuf2[0], ParmBuf2[1])) {
                  fprintf(stderr, "[%s]: ExcludeList - unsupported EMP" 
                     " \"%s\" or value \"%s\"\n",
                     ProgName, ExcludeList[i2].Fieldp->fieldName, ParmBuf2[1]);
                  ErrOpt=TRUE;                      /* put out a help message */
                  break;
             }
             i2++;                         /* incriment to next exclude field */
             ExcludeCount++;                  /* add 1 to exclude field count */
          }
          break;

       /*-------------------------------------------------*/
       /*- get a list of included fields and text        -*/
       /*-------------------------------------------------*/
       case   'i':
          SubParmCt=sscanf(optarg,
                           "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],"
                           "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],"
                           "%[^,],%[^,],%[^,],%[^,]",
                           ParmBuf[0],  ParmBuf[1],  ParmBuf[2],  ParmBuf[3],
                           ParmBuf[4],  ParmBuf[5],  ParmBuf[6],  ParmBuf[7],
                           ParmBuf[8],  ParmBuf[9],  ParmBuf[10], ParmBuf[11],
                           ParmBuf[12], ParmBuf[13], ParmBuf[14], ParmBuf[15],
                           ParmBuf[16], ParmBuf[17], ParmBuf[18], ParmBuf[19]);
          if (IncludeCount+SubParmCt > LIST_SZ) {
             fprintf(stderr, "[%s]: IncludeList list cannot exceed %d items!\n",
                ProgName, LIST_SZ);
             ErrOpt=TRUE;                           /* put out a help message */
             break;
          } 
          i2=IncludeCount;                /* next unused slot in Include list */
          for (i=0; i<(SubParmCt); i++)                  /* for each argument */
          {                             
             SubParmCt2=sscanf(ParmBuf[i], "%[^=]=%[^=]=%[^=]",
                               ParmBuf2[0], ParmBuf2[1], ParmBuf2[2]);
             if (SubParmCt2 != 2) {
                fprintf(stderr, "[%s]: Include sublist must have one \"=\","
                        " %i found\n",
                        ProgName, (SubParmCt2-1));
                ErrOpt=TRUE;                        /* put out a help message */
             }
             if (ValidateIE(&IncludeList[i2], ParmBuf2[0], ParmBuf2[1])) {
                  fprintf(stderr, "[%s]: IncludeList - unsupported EMP" 
                     " \"%s\" or value \"%s\"\n",
                     ProgName, IncludeList[i2].Fieldp->fieldName, ParmBuf2[1]);
                  ErrOpt=TRUE;                      /* put out a help message */
                  break;
             }
             i2++;                         /* incriment to next include field */
             IncludeCount++;                  /* add 1 to include field count */
          }
          break;

       /*-------------------------------------------------*/
       /*- get a list of selected fields                 -*/
       /*-------------------------------------------------*/
       case   's':
          SubParmCt=sscanf(optarg,
                           "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],"
                           "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],"
                           "%[^,],%[^,],%[^,],%[^,]",
                           ParmBuf[0],  ParmBuf[1],  ParmBuf[2],  ParmBuf[3],
                           ParmBuf[4],  ParmBuf[5],  ParmBuf[6],  ParmBuf[7],
                           ParmBuf[8],  ParmBuf[9],  ParmBuf[10], ParmBuf[11],
                           ParmBuf[12], ParmBuf[13], ParmBuf[14], ParmBuf[15],
                           ParmBuf[16], ParmBuf[17], ParmBuf[18], ParmBuf[19]);
          if (FieldCount+SubParmCt > LIST_SZ) {
             fprintf(stderr, "[%s]: Select list cannot exceed %d items!\n",
                ProgName, LIST_SZ);
             ErrOpt=TRUE;                           /* put out a help message */
             break;
          }  else {                             
             for (i=0; i<SubParmCt; i++)             /* for each sub argument */
             {                             
                if (isdigit(ParmBuf[i][0]))            /* if a digit, take it */
                {
                  empWork=atoi(ParmBuf[i]);
                  if (empWork == 0) {       /* emp 0 (user data) is imprecise */
                     fprintf(stderr,
                        "[%s]: FieldList - userdata emp 0 is imprecise\"%s\"\n",
                        ProgName);
                     ErrOpt=TRUE;                   /* put out a help message */
                     break;
                  }
                } else {                  /* else, convert name to emp number */
                  empWork=Field2Num(&ParmBuf[i][0]);
                  if (empWork < 0) {
                     fprintf(stderr,
                        "[%s]: FieldList - unrecognized emp \"%s\"\n",
                        ProgName, &ParmBuf[i][0]);
                     ErrOpt=TRUE;                   /* put out a help message */
                     break;
                  }
                }                               /* setup pointer to FieldDesc */
                FieldList[FieldCount].Fieldp =
                     emp2Fieldp(empWork, &ParmBuf[i][0]);
                ReportWidth =                            /* add to line width */
                     ReportWidth+1+FieldList[FieldCount].Fieldp->outSize;
                FieldCount++;                /* add 1 to selected field count */
                if (FieldCount > LIST_SZ) {
                   fprintf(stderr, "[%s]: FieldCount - more than %i entries\n",
                      ProgName, LIST_SZ);
                   ErrOpt=TRUE;                     /* put out a help message */
                   break;
                } 
             } 
          } 
          break;

       /*-------------------------------------------------*/
       /*- put out a help message                        -*/
       /*-------------------------------------------------*/
       case   '?':
          ErrOpt=TRUE;
          break;

       } /* end switch option */
    } /* end while option */

    if (! FieldCount) {                         /* if no user selected fields */
       /*--------------------------------*/
       /* Setup default selection fields */
       /*--------------------------------*/
       FieldList[0].Fieldp=emp2Fieldp(1, NULL);             /* Transaction Id */
       FieldList[1].Fieldp=emp2Fieldp(2, NULL);                /* Terminal Id */
       FieldList[2].Fieldp=emp2Fieldp(89, NULL);                   /* User Id */
       FieldList[3].Fieldp=emp2Fieldp(5, NULL);                 /* Start Time */
       FieldList[4].Fieldp=emp2Fieldp(7, NULL);               /* Elapsed Time */
       FieldCount=5;                             /* 5 default fields selected */
    }

    if (SummaryRF && !ErrOpt) {
       /*------------------------------------------------------------*/
       /* validate that the select list is valid for summary reports */
       /* - ie. no time stamp fields & only maj/min string fields    */
       /*------------------------------------------------------------*/
       int validMajor = FALSE;           /* validate that the major and minor */
       int validMinor = FALSE;              /* columns are in the select list */

       for(i = 0; i < FieldCount; i++) {
          if (FieldList[i].Fieldp->empNum == Table.Major->empNum)
             validMajor = TRUE;                         /* major column found */
          if (Table.Minor != NULL)       /* minor column is an optional field */
             if (FieldList[i].Fieldp->empNum == Table.Minor->empNum)
                validMinor = TRUE;                      /* minor column found */
          switch (FieldList[i].Fieldp->fieldType) {
             case CICS_EMP_TDQ_DETAIL_TYPE_TIME_STAMP:
             case CICS_EMP_TDQ_DETAIL_TYPE_USER:
                  fprintf(stderr, "[%s]: Select list includes an invalid "
                          "data type (%s) for summary reports\n", ProgName,
                          FieldList[i].Fieldp->fieldName);
                  ErrOpt=TRUE;
                  break;
             case CICS_EMP_TDQ_DETAIL_TYPE_STRING:
             case CICS_EMP_TDQ_DETAIL_TYPE_ABEND_CODE:
                  if ((FieldList[i].Fieldp->empNum != Table.Major->empNum) &&
                     (FieldList[i].Fieldp->empNum != Table.Minor->empNum)) {
                     fprintf(stderr, "[%s]: Select list includes a string "
                          "data type (%s) which is not a major/minor "
                          "summary field\n", ProgName,
                          FieldList[i].Fieldp->fieldName);
                     ErrOpt=TRUE;
                  }
                  break;
             case CICS_EMP_TDQ_DETAIL_TYPE_COUNTER:
             case CICS_EMP_TDQ_DETAIL_TYPE_OS_COUNT:
                  if ((FieldList[i].Fieldp->empNum == CICS_EMP_FID_TCASEQ_NO) ||
                     (FieldList[i].Fieldp->empNum == CICS_EMP_FID_TASK_FLAGS)) {
                     fprintf(stderr, "[%s]: Select list includes an invalid "
                             "data type (%s) for summary reports\n", ProgName,
                             FieldList[i].Fieldp->fieldName);
                     ErrOpt=TRUE;
                  }
                  break;
             default:
                  break;
          }
       }
       if (validMajor == FALSE) {
          fprintf(stderr, "[%s]: Major column (%s) must be in the select list"
                          "\n", ProgName, Table.Major->fieldName);
          ErrOpt=TRUE;
       }
       if ((validMinor == FALSE) && (Table.Minor != NULL)) {
          fprintf(stderr, "[%s]: Minor column (%s) must be in the select list"
                          "\n", ProgName, Table.Minor->fieldName);
          ErrOpt=TRUE;
       }
    } /* end if (SummaryRF) */

    /*-------------------------------------------------*/
    /*- somthing wrong, put out a help message        -*/
    /*-------------------------------------------------*/
    if (ErrOpt || ! FileNameProcessed)
    {
       fprintf(stderr, "Usage: %s -f <cmf-input-file>\n", ProgName);
       fprintf(stderr, "       [-t <user-title-text>]\n");
       fprintf(stderr, "       [-l <lines-per-page> {%d}]\n", PageLth);
       fprintf(stderr, "       [-w <chars-per-line> {%d}]\n", LineLth);
       fprintf(stderr, "       [-T {suppress titles}]\n");
       fprintf(stderr, "       [-q {suppress final totals}]\n");
       fprintf(stderr, "       [-x {suppress detail lines}]\n");
       fprintf(stderr, "       [-G {print timestamps in GMT}]\n");
       fprintf(stderr, "       [-S {print timer values in sec.ms}]\n");
       fprintf(stderr, "       [-c {print counters with clocks}]\n");
       fprintf(stderr, "       [-s <n,n,n,...>]           "
                               "list of selected fields\n");
       fprintf(stderr, "       [-i <n=value,n=value,...>] "
                               "list of included field values\n");
       fprintf(stderr, "       [-e <n=value,n=value,...>] "
                               "list of excluded field values\n");
       fprintf(stderr, "       [-a <major,minor>]         "
                               "request (accumulate) a summary report\n");
       fprintf(stderr, "  Field_Id = emp_number -or- name accepted\n");
       for (i=0; i<MonFields; i++)      /* show all valid fields in FormatMap */
       {
           fprintf(stderr, "   %3.3i %-12.12s",
              FormatMap[i].empNum, FormatMap[i].fieldName);
           if ((i+1) % 4 == 0)
              fprintf(stderr, "\n");
       }
       fprintf(stderr, "\n");

#ifdef DEBUG
       fprintf(stderr, "argc(%d), argv: ", argc);
       for (i=0; i<argc; i++)                         /* for each argument */
          fprintf(stderr, "%s ", argv[i]);
       fprintf(stderr, "\n");
#endif

       return(1);
    } /* end if (ErrOpt) */

    PageLth-=TitleLines;             /* subtract title lines from page length */
    return(0);
} /* end cmfopts() */

/******************************************************************************/
/* ValidateIE()                                 validate include/exclude item */
/******************************************************************************/
int ValidateIE(struct ExcIncList_t *ExcIncItem, char *argemp, char *argvalue)
{
  int                 i, i2, EmpWork, TimeValCt;             /* work integers */
  long int            ArgvInt=0;                     /* argument value as int */
  struct FieldDesc_t *ReportField;        /* the report description structure */
  char                HourValue[9]={'\0'};                   /* sub arguments */
  char                MinuteValue[9]={'\0'};                 /* sub arguments */
  char                SecondValue[9]={'\0'};                 /* sub arguments */
  char                JunqueValue[9]={'\0'};                 /* sub arguments */

   /*-------------------------------------------------*/
   /*- determine which emp is to be compared         -*/
   /*-------------------------------------------------*/
   if (isdigit(argemp[0]))                             /* if a digit, take it */
   {
     EmpWork=atoi(argemp);
   } else {                     /* else, convert name to emp number */
     EmpWork=Field2Num(argemp);
     if (EmpWork < 0) {
        fprintf(stderr,
           "[%s]: ExcludeList - unrecognized emp \"%s\"\n",
           ProgName, argemp);
        return(1);                                  /* put out a help message */
     }
   }
   ExcIncItem->Fieldp = emp2Fieldp(EmpWork, argemp);          /* point to emp */
   strcpy(ExcIncItem->text, argvalue);            /* text comparison argument */
   ReportField=ExcIncItem->Fieldp;     /* pointer to report field description */
#ifdef DEBUG
   fprintf(stderr, "[%s]: Processing %s=%s\n", ProgName, argemp, argvalue);
#endif

   /*-------------------------------------------------*/
   /*- determine what comparison operator is desired -*/
   /*-------------------------------------------------*/
   i2=strlen(ExcIncItem->text);                 /* get the value field length */
   if (i2 < 1) return(1);                 /* comparison data must be included */
   if      (! memcmp(ExcIncItem->text, "<", 1))
            ExcIncItem->oper=LESSTHAN;
   else if (! memcmp(ExcIncItem->text, ">", 1))
            ExcIncItem->oper=GREATERTHAN;
   else
            ExcIncItem->oper=EQUALS;
   if      (! memcmp(ExcIncItem->text, "<", 1) ||     /* if compare operator, */
            ! memcmp(ExcIncItem->text, ">", 1))
       strcpy(&ExcIncItem->text[0], &ExcIncItem->text[1]);       /* remove it */
   i2=strlen(ExcIncItem->text);       /* get the remaining value field length */
   if (i2 < 1) return(1);            /* more comparison data must be included */

   /*------------------------------------------*/
   /*- if a numeric value or hh:mm:ss  value, -*/
   /*- convert it to an integer               -*/
   /*------------------------------------------*/
   if (ReportField->fieldType != CICS_EMP_TDQ_DETAIL_TYPE_STRING)
   {
      for (i=0; i<i2; i++)
          if ((! isdigit(ExcIncItem->text[i])) &&
              (memcmp(&ExcIncItem->text[i], ":", 1)))
              return(1);                    /* if not a digit or colon, error */
      TimeValCt=sscanf(ExcIncItem->text, "%[^:]:%[^:]:%[^:]:%[^:]",
                       HourValue, MinuteValue, SecondValue, JunqueValue);
      if ((TimeValCt > 3) || (TimeValCt < 1))
         return(1);                              /* invalid colon combination */
      if (TimeValCt == 1)
         ArgvInt = atoi(ExcIncItem->text);             /* it's numeric only ! */
      if (TimeValCt == 3)                                         /* hh:mm:ss */
      {
         i = atoi(HourValue);
         if ((i < 24) && (i > -1))
            ArgvInt = i * 3600;                                      /* hours */
         else
              return(1);
      }
      if (TimeValCt > 1)                                             /* mm:ss */
      {
         i = atoi(MinuteValue);
         if ((i < 60) && (i > -1))
            ArgvInt = ArgvInt + (i * 60);                          /* minutes */
         else
              return(1);
         i = atoi(SecondValue);
         if ((i < 60) && (i > -1))
            ArgvInt = ArgvInt + i;                                 /* seconds */
         else
              return(1);
      }
#ifdef DEBUG
      fprintf(stderr, "[%s]: Processed(%i) %s %s [%i]\n",
                 ProgName, TimeValCt, argemp, ExcIncItem->text, ArgvInt);
#endif
   }

   /*---------------------------------------------------*/
   /*- now, set the comparison value based on emp type -*/
   /*---------------------------------------------------*/
   switch (ReportField->fieldType)
   {
      case CICS_EMP_TDQ_DETAIL_TYPE_STRING:       /* string compares are fine */
         break;
      case CICS_EMP_TDQ_DETAIL_TYPE_OS_COUNT:
      case CICS_EMP_TDQ_DETAIL_TYPE_COUNTER:
      case CICS_EMP_TDQ_DETAIL_TYPE_TIME_STAMP:
      case CICS_EMP_TDQ_DETAIL_TYPE_OS_CLOCK:
      case CICS_EMP_TDQ_DETAIL_TYPE_TIMER:
         ExcIncItem->value = ArgvInt;   /* accept the converted numeric value */
         break;
      default:                             /* all other EMP types are invalid */
         return(1);
   }

   return(0);
} /* end ValidateIE() */

/******************************************************************************/
/* emp2Fieldp()   Returns a pointer to the report descriptor for an emp field */
/******************************************************************************/
struct FieldDesc_t *emp2Fieldp(cics_ulong_t EMP, char *textStr)
{
  int      i;                                                /* work integers */

  for (i=0; i<MonFields; i++)   /* find the report descriptor from emp number */
      if (EMP == CICS_EMP_FID_USER_SPACE)
      {                                 /* user emp has sub-types to consider */
          if (EMP == FormatMap[i].empNum
             && ! strcmp(textStr, FormatMap[i].fieldName))
              return(&FormatMap[i]);
      }
      else
      {                           /* all others have unique field descriptors */
          if (EMP == FormatMap[i].empNum)
              return(&FormatMap[i]);
      }

  return(NULL);                                           /* default to error */
} /* end emp2Fieldp() */

/******************************************************************************/
/* Field2Num()                      Convert text field name to CMF emp number */
/******************************************************************************/
int Field2Num(char *textStr)
{
  int      i;                                                /* work integers */

  for (i=0; i<MonFields; i++)          /* find the field number in FormatMap */
      if (! strcmp (FormatMap[i].fieldName, textStr))
          return(FormatMap[i].empNum);

  return(-1);                                      /* empNum '-1' is an error */
} /* end Field2Num() */

