#!/bin/ksh
################################################################################
# sasmpleaix.cmd                                               Rags Int., Inc. #
#               sample syntax for cmfreport listings                           #
#                                                                              #
#  Rags Int., Inc.                                              December 1999  #
#  (c) RISC Application Growth Solutions International, Incorporated  1999     #
#              (817) 267-2554       rags@compuserve.com                        #
#                                                                              #
################################################################################
typeset    SCRIPT=${0##*/};                # this script name
typeset    CMFFILE=${1:-"ragsinc.dat"};    # CMF Data Input File
typeset -i CASENO=0;                       # Test Case Number

if [ "${CMFFILE}" = "" ];then
   print "[${SCRIPT}]: You must specify an input file name";
   exit 1;
fi

if [ ! -f  "${CMFFILE}" ];then
   print "[${SCRIPT}]: File \"${CMFFILE}\" does not exist";
   exit 1;
fi
print "[${SCRIPT}]: Processing File \"${CMFFILE}\"";

#-----------------------------------------------------------------------#
eval CASENO=${CASENO}+1;
print "[${SCRIPT}]: Test Case ${CASENO} - Default Options";
./cmfreport -t "Test Case ${CASENO} - Default Options" \
            -f ${CMFFILE} > ${CMFFILE}.fmt${CASENO};
#-----------------------------------------------------------------------#
eval CASENO=${CASENO}+1;
print "[${SCRIPT}]: Test Case ${CASENO} - Exclude CICS Transactions";
./cmfreport -w 132 \
            -s TranId,TranType,StartTime,ElapsedTime \
            -s SYNCPOINT,FILE_ADD,CICStime,CICS_CPU,UserCPU,KernelCPU \
            -e TranId=C \
            -t "Test Case ${CASENO} - Exclude CICS Transactions" \
            -c \
            -f ${CMFFILE} > ${CMFFILE}.fmt${CASENO};
#-----------------------------------------------------------------------#
eval CASENO=${CASENO}+1;
print "[${SCRIPT}]: Test Case ${CASENO} - Include User SYSAD";
./cmfreport -w 132 \
            -s TranId,TranType,StartTime,ElapsedTime \
            -s TD_IO_Time,TS_IO_Time,FILE_Time,FILE_Total,LoadTime \
            -s CtxSwVol,CtxSwInv \
            -i UserId=SYSAD \
            -t "Test Case ${CASENO} - Include User SYSAD" \
            -f ${CMFFILE} > ${CMFFILE}.fmt${CASENO};
#-----------------------------------------------------------------------#
eval CASENO=${CASENO}+1;
print "[${SCRIPT}]: Test Case ${CASENO} - Major TranId / Minor UserId";
./cmfreport -s TranId,UserId,ElapsedTime \
            -s UserCPU,SYNCPOINT,TD_PUT \
            -a TranId,UserId \
            -e TranId=C \
            -t "Test Case ${CASENO} - Major TranId / Minor UserId" \
            -f ${CMFFILE} > ${CMFFILE}.fmt${CASENO};
#-----------------------------------------------------------------------#
eval CASENO=${CASENO}+1;
print "[${SCRIPT}]: Test Case ${CASENO} - Major UserId / Minor TranId";
./cmfreport -s TranId,UserId,ElapsedTime \
            -s SYNCPOINT,BMS_MAP,UserCPU,CICStime \
            -a UserId,TranId \
            -e TranId=C \
            -t "Test Case ${CASENO} - Major UserId / Minor TranId" \
            -c \
            -f ${CMFFILE} > ${CMFFILE}.fmt${CASENO};
#----------------------------------------------------------------------#
eval CASENO=${CASENO}+1;
print "[${SCRIPT}]: Test Case ${CASENO} - Major UserId";
./cmfreport -s UserId,ElapsedTime,FILE_Time \
            -s SYNCPOINT,BMS_MAP,UserCPU,CICStime \
            -a UserId \
            -e TranId=C \
            -t "Test Case ${CASENO} - Major UserId" \
            -c \
            -f ${CMFFILE} > ${CMFFILE}.fmt${CASENO};
#----------------------------------------------------------------------#
eval CASENO=${CASENO}+1;
print "[${SCRIPT}]: Test Case ${CASENO} - Include based on T.O.D.";
./cmfreport -s UserId,ElapsedTime,FILE_Time \
            -s SYNCPOINT,BMS_MAP,UserCPU,CICStime \
            -a UserId \
            -e TranId=C \
            -e "StartTime=<12:00:00,StartTime=>13:59:59" \
            -t "Test Case ${CASENO} - Include based on T.O.D." \
            -f ${CMFFILE} > ${CMFFILE}.fmt${CASENO};
#----------------------------------------------------------------------#
eval CASENO=${CASENO}+1;
print "[${SCRIPT}]: Test Case ${CASENO} - Major UserId / Minor PROGNAME";
./cmfreport -s TranId,ProgName,ElapsedTime \
            -s SYNCPOINT,BMS_MAP,UserCPU,CICStime \
            -a UserId,ProgName \
            -e TranId=C \
            -t "Test Case ${CASENO} - Major UserId / Minor PROGNAME" \
            -c \
            -f ${CMFFILE} > ${CMFFILE}.fmt${CASENO};
#----------------------------------------------------------------------#

exit $?;

